package VDRTools::Config;

use strict;
use VDRTools qw(error);
use VDRTools::File;

my $ERRORS = {
	NOPARSE		=> 'Could not parse file "%s"',
	NODO		=> 'Could not do file "%s"',
	NORUN		=> 'Could not run file "%s"',
	NOWRITE		=> 'Can not write file "%s"',
};


######################################################################

sub new {
	my $self = shift;
	my $path2config = shift || '';
	
	unless ( $path2config ) {
		error( 'No config file defined', 'VDRTools::Config::new()' );
		return undef;
	}
	
	$path2config = vt_absolute( $path2config );
	
	my $ref;
	
	if ( -e $path2config ) {
		$ref = do $path2config;
		if ( $@ ) {
			error( sprintf( $ERRORS->{NOPARSE}, $path2config ), 'VDRTools::Config::new()' );
			return undef;
		}
		unless ( defined( $ref ) ) {
			error( sprintf( $ERRORS->{NODO}, $path2config ), 'VDRTools::Config::new()' );
			return undef;
		}
		unless ( $ref ) {
			error( sprintf( $ERRORS->{NORUN}, $path2config ), 'VDRTools::Config::new()' );
			return undef;
		}
	}
	else {
		$ref = {};
	}
	$ref->{'path2config'} = $path2config if ( $path2config );
	
	bless($ref,$self);
	
	return $ref;
}

sub save {
	my $self = shift;
	
	my $data = '{' . "\n";
	
	for my $key ( keys ( %$self ) ) {
		$data .= _parse_key( 1, $key, $self->{$key} );
	}
	
	$data .= '};' . "\n";
	local *FILE;
	unless ( open ( FILE, ">" . $self->{'path2config'} ) ) {
		error( sprintf( $ERRORS->{NOWRITE}, $self->{'path2config'} ), 'VDRTools::Config->save()' );
		return;
	}
	open ( FILE, ">" . $self->{'path2config'} );
	print FILE $data;
	close FILE;
	return 1;
}

sub _parse_value {
	my ( $tabs, $value ) = @_;
	my $return;
	
	if ( not ref ( $value ) ) {
		$return = '\'' . _escape ( $value ) . '\',' . "\n";
	}
	elsif ( ref( $value ) eq 'SCALAR' ) {
		$return = '\'' . _escape ( $$value ) . '\',' . "\n";
	}
	elsif ( ref ( $value ) eq 'HASH' ) {
		$return = '{' . "\n";
		for my $key ( keys ( %$value ) ) {
			$return .= _parse_key( $tabs + 1, $key, $value->{$key} );
		}
		$return .= "\t" x $tabs . '},' . "\n";
	}
	elsif ( ref ( $value ) eq 'ARRAY' ) {
		$return = '[' . "\n";
		foreach ( @$value ) {
			$return .= "\t" x ($tabs + 1) . _parse_value ( $tabs + 1, $_ );
		}
		$return .= "\t" x $tabs . '],' . "\n";
	}
}

sub _parse_key {
	my $tabs = shift;
	my $key = shift;
	my $value = shift;
	my $return;
	
	$return = "\t" x $tabs . '\'' . $key . '\' => ' . _parse_value ( $tabs, $value );
	
	return $return;
}

sub _escape {
    my $value = shift;
    defined($value) or return 'undef';
    $value =~ s/('|\\(?=['\\]|$))/\\$1/g;
    return $value;
}

1;

__END__

=pod

=head1 NAME

VDRTools::Config

=head1 SYNOPSIS

=over 4

use VDRTools::Config;

my $cfg = VDRTools::Config->new( '/path/to/config/file' );

$cfg->{scalar} = $scalar;

$cfg->{scalar} = \$scalar;

$cfg->{arrayref} = \@array;

$cfg->{hashref} = \%hash;

$cfg->save();

=cut
