package VDRTools::Network;

use strict;
use vars qw(
	@ISA
	@EXPORT_OK
	$ERRORS
);

require Exporter;

@ISA = qw(Exporter);
@EXPORT_OK = qw(
	vt_svdrpsend
);

use Socket;
use VDRTools qw(error);
use VDRTools::System;

$ERRORS = {
	SOCKETOPEN	=> 'Can not open socket',
	NOARGS		=> 'No arguments passed to function',
	BADOPTS		=> 'Unknown option %s passed to function',
	SOCKETCONNECT	=> 'Can not connect to socket',
};

######################################################################

sub vt_svdrpsend {
	my ( $cmd, $opts ) = @_;
	
	$opts = {} unless ( $opts and ref $opts eq 'HASH' );
	
	unless ( $cmd ) {
		error( $ERRORS->{NOARGS}, 'VDRTools::Network::vt_svdrpsend()' );
		return;
	}
	
	my $hostname = delete( $opts->{hostname} );
	$hostname = 'localhost' unless ( defined( $hostname ) );
	
	my $port = delete( $opts->{port} );
	$port = 2001 unless ( defined( $port ) );
	
	my $timeout = delete( $opts->{timeout} );
	$timeout = 10 unless ( defined( $timeout ) );
	
	if ( keys( %$opts ) ) {
		error( sprintf( $ERRORS->{BADOPTS}, ( join ', ', keys( %$opts ) ) ), 'VDRTools::Network::vt_svdrpsend()' );
		return;
	}
	
	my $oldalrm = $SIG{ALRM};
	$SIG{ALRM} = sub { error( 'timeout', 'VDRTools::Network::vt_svdrpsend()' ); };
	alarm( $timeout );
	
	my $iaddr = inet_aton( $hostname ) || error( 'no host: ' . $hostname, 'VDRTools::Network::vt_svdrpsend()' );
	my $paddr = sockaddr_in( $port, $iaddr );
	
	my $proto = getprotobyname( 'tcp' );
	
	local *SOCK;
	
	unless ( socket( SOCK, PF_INET, SOCK_STREAM, $proto ) ) {
		error( $ERRORS->{SOCKETOPEN}, 'VDRTools::Network::vt_svdrpsend()' );
		$SIG{ALRM} = $oldalrm;
		alarm(0);
		return;
	}
	
	unless ( connect( SOCK, $paddr ) ) {
		error( $ERRORS->{SOCKETCONNECT}, 'VDRTools::Network::vt_svdrpsend()' );
		close( SOCK );
		$SIG{ALRM} = $oldalrm;
		alarm(0);
		return;
	}
	
	select( SOCK );
	$| = 1;
	
	while ( <SOCK> ) {
		my ( $ret ) = $_ =~ /^(\d{3}) /;
		unless ( $ret eq '220' ) {
			error( 'VDR service not ready', 'VDRTools::Network::vt_svdrpsend()' );
			close( SOCK );
			$SIG{ALRM} = $oldalrm;
			alarm(0);
			return $ret;
		}
		last;
	}
	
	print SOCK $cmd . "\r\n";
	
	my $g_ret;
	
	while ( <SOCK> ) {
		my ( $ret ) = $_ =~ /^(\d{3}) /;
		unless ( $ret eq '250' ) {
			error( 'Requested VDR action not okay', 'VDRTools::Network::vt_svdrpsend()' );
			close( SOCK );
			$SIG{ALRM} = $oldalrm;
			alarm(0);
			return $ret;
		}
		$g_ret = $ret;
		last;
	}
	
	print SOCK "quit\r\n";
	
	while ( <SOCK> ) {
		my ( $ret ) = $_ =~ /^(\d{3}) /;
		unless ( $ret eq '221' ) {
			error( 'VDR service closing transmission channel failed', 'VDRTools::Network::vt_svdrpsend()' );
			close( SOCK );
			$SIG{ALRM} = $oldalrm;
			alarm(0);
			return $ret;
		}
		last;
	}
	
	close( SOCK );
	$SIG{ALRM} = $oldalrm;
	alarm(0);
	
	return $g_ret;
}

1;
