package VDRTools::Sysconfig;

use strict;
use VDRTools qw(error);
use VDRTools::File;
use POSIX qw(strftime);

my $ERRORS = {
	NOREAD		=> 'Can not read file "%s"',
	NOWRITE		=> 'Can not write file "%s"',
};

######################################################################

sub new {
	my $self = shift;
	
	my $ref;
	
	if ( -e $VDRTools::SYSCONFIG ) {
		local *SYSCONFIG;
		unless ( open( SYSCONFIG, $VDRTools::SYSCONFIG ) ) {
			error( sprintf( $ERRORS->{NOREAD}, $VDRTools::SYSCONFIG ), 'VDRTools::Sysconfig->new()' );
			return undef;
		}
		while( <SYSCONFIG> ) {
			next if(/^#/);   
			/(.*)="(.*)"/;
			my( $key, $value ) = ( $1, $2 );
			$ref->{$key} = $value;
		}
		close( SYSCONFIG );
		$ref = {} unless ( defined( $ref ) );
	}
	else {
		return undef;
	}
	
	bless( $ref, $self );
	
	return $ref;
}

sub save {
	my $self = shift;
	
	my ( $sec, $min, $hour, $day, $month, $year ) = localtime( time() );
	
	my $header = sprintf( "#\n# generated by VDRTools::Sysconfig at %s\n#\n", strftime( "%d.%m.%Y %H:%M:%S", localtime( time() ) ) );
	
	my $data = '';
	
	for my $key ( sort( keys( %$self ) ) ) {
		if( $key ) {
			$data .= "$key=\"" . $self->{$key} . "\"\n";
		}
	}
	
	local *SYSCONFIG;
	unless ( open ( SYSCONFIG, ">" . $VDRTools::SYSCONFIG ) ) {
		error( sprintf( $ERRORS->{NOWRITE}, $VDRTools::SYSCONFIG ), 'VDRTools::Sysconfig->save()' );
		return;
	}
	print SYSCONFIG $header;
	print SYSCONFIG $data;
	close( SYSCONFIG );
	return 1;
}

1;

__END__

=pod

=head1 NAME

VDRTools::Sysconfig

=head1 SYNOPSIS

=over 4

use VDRTools::Sysconfig;

my $syscfg = VDRTools::Sysconfig->new();

$syscfg->{scalar} = $scalar;

$syscfg->save();

=cut
